@@ -1,30 +1,19 @@
+import json
 import os
 import sys
-import json
+from typing import Dict, List, Union
 
 DEFAULT_DB = os.path.join(os.path.dirname(
     __file__), 'data', 'const-default.json')
 
 
 class Constants():
-    """Interface for constants
-    """
+    """Interface for constants"""
 
-    LEGACY_MAPPING = {
-        'known_machines': 'replacements/machines',
-        'known_distros': 'replacements/distros',
-        'known_mirrors': 'replacements/mirrors',
-        'known_vars': 'variables/known',
-        'mandatory_vars': 'variables/mandatory',
-        'protected_append_vars': 'variables/protected-append',
-        'protected_vars': 'variables/protected',
-        'suggested_vars': 'variables/suggested',
-    }
-
-    def __init__(self):
+    def __init__(self) -> None:
         self.__db = self.__load_db(DEFAULT_DB)
 
-    def __load_db(self, path):
+    def __load_db(self, path: str) -> dict:
         try:
             with open(path) as _in:
                 return json.load(_in)
@@ -32,215 +21,197 @@ class Constants():
             sys.stderr.write('Cannot load constant database\n')
             return {}
 
-    def __get_from_path(self, path):
+    def GetByPath(self, path: str) -> Union[Dict, List]:
+        """Get constant from path
+
+        Args:
+            path (str): / joined path in the constant structure
+
+        Returns:
+            Union[Dict, List]: Item in structure or empty dictionary
+        """
         paths = path.rstrip('/').split('/')
         data = self.__db
         for _, value in enumerate(paths):
             data = data.get(value, {})
         return data
 
-    def AddConstants(self, _dict):
+    def AddConstants(self, _dict: dict) -> None:
         """Add constants to the existing
 
         Args:
             dict (dict): constant dictionary to add
         """
-        def dict_merge(a, b):
+        def dict_merge(a: dict, b: dict) -> dict:
             for k, v in b.items():
-                if isinstance(b[k], dict):
-                    dict_merge(a[k], b[k])
-                elif k not in a:
+                if k not in a:
                     a[k] = v
+                elif isinstance(b[k], dict):
+                    dict_merge(a[k], b[k])
                 else:
                     a[k] += v
             return a
         self.__db = dict_merge(self.__db, _dict)
 
-    def RemoveConstants(self, _dict):
+    def RemoveConstants(self, _dict: dict) -> None:
         """Remove constants from the existing
 
         Args:
             dict (dict): constant dictionary to remove
         """
-        def dict_merge(a, b):
+        def dict_merge(a: dict, b: dict) -> dict:
             for k, v in b.items():
-                if isinstance(b[k], dict):
-                    dict_merge(a[k], b[k])
-                elif k not in a:
+                if k not in a:
                     pass
+                elif isinstance(b[k], dict):
+                    dict_merge(a[k], b[k])
                 else:
                     a[k] = [x for x in a[k] if x not in v]
             return a
         self.__db = dict_merge(self.__db, _dict)
 
-    def OverrideConstants(self, _dict):
+    def OverrideConstants(self, _dict: dict) -> None:
         """Override constants in the existing db
 
         Args:
             dict (dict]): constant dictionary with override values
         """
-        def dict_merge(a, b):
+        def dict_merge(a: dict, b: dict) -> dict:
             for k, v in b.items():
-                if isinstance(b[k], dict):
+                if k not in a:
+                    a[k] = v
+                elif isinstance(b[k], dict):
                     dict_merge(a[k], b[k])
                 else:
                     a[k] = v
             return a
         self.__db = dict_merge(self.__db, _dict)
 
-    def AddFromRuleFile(self, _dict):
-        """Legacy interface to support rule files
-
-        Args:
-            dict (dict): rule file dictionary
-        """
-        def dict_nested_set(d, path, value):
-            crumb = path[0]
-            if len(path) == 1:
-                d[crumb] = value
-            else:
-                if crumb not in d:
-                    d[crumb] = {}
-                dict_nested_set(d[crumb], path[1:], value)
-        _translated = {}
-        for n, r in Constants.LEGACY_MAPPING.items():
-            if n in _dict:
-                dict_nested_set(_translated, r.split('/'), _dict[n])
-        self.AddConstants(_translated)
-
-    def AddFromConstantFile(self, _dict):
-        """Legacy interface to support constant files
-
-        Args:
-            dict (dict): constant file dictionary
-        """
-        self.AddFromRuleFile(_dict)
-
     @property
-    def FunctionsKnown(self):
+    def FunctionsKnown(self) -> List[str]:
         """Return known functions
 
         Returns:
             list: list of known functions
         """
-        return self.__get_from_path('functions/known')
+        return self.GetByPath('functions/known')
 
     @property
-    def FunctionsOrder(self):
+    def FunctionsOrder(self) -> List[str]:
         """Return function order
 
         Returns:
             list: List of functions to order in their designated order
         """
-        return self.__get_from_path('functions/order')
+        return self.GetByPath('functions/order')
 
     @property
-    def VariablesMandatory(self):
+    def VariablesMandatory(self) -> List[str]:
         """Return mandatory variables
 
         Returns:
             list: List of mandatory variables
         """
-        return self.__get_from_path('variables/mandatory')
+        return self.GetByPath('variables/mandatory')
 
     @property
-    def VariablesSuggested(self):
+    def VariablesSuggested(self) -> List[str]:
         """Return suggested variables
 
         Returns:
             list: List of suggested variables
         """
-        return self.__get_from_path('variables/suggested')
+        return self.GetByPath('variables/suggested')
 
     @property
-    def MirrorsKnown(self):
+    def MirrorsKnown(self) -> Dict[str, str]:
         """Return known mirrors and their replacements
 
         Returns:
             dict: Dict of known mirrors and their replacements
         """
-        return self.__get_from_path('replacements/mirrors')
+        return self.GetByPath('replacements/mirrors')
 
     @property
-    def VariablesProtected(self):
+    def VariablesProtected(self) -> List[str]:
         """Return protected variables
 
         Returns:
             list: List of protected variables
         """
-        return self.__get_from_path('variables/protected')
+        return self.GetByPath('variables/protected')
 
     @property
-    def VariablesProtectedAppend(self):
+    def VariablesProtectedAppend(self) -> List[str]:
         """Return protected variables in bbappend files
 
         Returns:
             list: List of protected variables in bbappend files
         """
-        return self.__get_from_path('variables/protected-append')
+        return self.GetByPath('variables/protected-append')
 
     @property
-    def VariablesOrder(self):
+    def VariablesOrder(self) -> List[str]:
         """Variable order
 
         Returns:
             list: List of variables to order in their designated order
         """
-        return self.__get_from_path('variables/order')
+        return self.GetByPath('variables/order')
 
     @property
-    def VariablesKnown(self):
+    def VariablesKnown(self) -> List[str]:
         """Known variables
 
         Returns:
             list: List of known variables
         """
-        return self.__get_from_path('variables/known')
+        return self.GetByPath('variables/known')
 
     @property
-    def DistrosKnown(self):
+    def DistrosKnown(self) -> List[str]:
         """Known distros
 
         Returns:
             list: List of known distros
         """
-        return self.__get_from_path('replacements/distros')
+        return self.GetByPath('replacements/distros')
 
     @property
-    def MachinesKnown(self):
+    def MachinesKnown(self) -> List[str]:
         """Known machines
 
         Returns:
             list: List of known machines
         """
-        return self.__get_from_path('replacements/machines')
+        return self.GetByPath('replacements/machines')
 
     @property
-    def ImagesClasses(self):
+    def ImagesClasses(self) -> List[str]:
         """Classes that are used in images
 
         Returns:
             list: Classes that are used in images
         """
-        return self.__get_from_path('images/known-classes')
+        return self.GetByPath('images/known-classes')
 
     @property
-    def ImagesVariables(self):
+    def ImagesVariables(self) -> List[str]:
         """Variables that are used in images
 
         Returns:
             list: Variables that are used in images
         """
-        return self.__get_from_path('images/known-variables')
+        return self.GetByPath('images/known-variables')
 
     @property
-    def SetsBase(self):
+    def SetsBase(self) -> Dict[str, str]:
         """Base variable set
 
         Returns:
             dict: dictionary with base variable set
         """
-        return self.__get_from_path('sets/base')
+        return self.GetByPath('sets/base')
 
 
 CONSTANTS = getattr(sys.modules[__name__], 'CONSTANTS', Constants())
